package edu.chl.hajo.jeesec.db;



import java.util.ArrayList;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

/**
 * Base class for ProductCatalogur, OrderBook, CustomerRefistry
 *
 * @author hajo
 */
public class AbstractDAO<T, K> implements IDAO<T, K> {
   
    private EntityManager em;
    private Class<T> clazz;

    public AbstractDAO() {
    }
   
    protected AbstractDAO(Class<T> clazz) {
        this.clazz = clazz;
    }

    protected EntityManager getEntityManager() {
        return em;
    }

    protected void setEntitymanager(EntityManager em) {
        this.em = em;
    }
    
    @Override
    public void add(T t) {
        em.persist(t);
    }

    @Override
    public void remove(K id) {
        T t = em.getReference(clazz, id);
        em.remove(t);
    }

    /**
     * Using out parameter
     */
    @Override
    public void update(T t) {
        em.merge(t);
    }

    @Override
    public T find(K id) {
        return em.find(clazz, id);
    }

    @Override
    public List<T> getAll() {
        return findEntities(true, -1, -1);
    }

    @Override
    public List<T> getRange(int maxResults, int firstResult) {
        return findEntities(false, maxResults, firstResult);
    }

    @Override
    public int getCount() {
        int count = -1;
        CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
        Root<T> rt = cq.from(clazz);
        cq.select(em.getCriteriaBuilder().count(rt));
        Query q = em.createQuery(cq);
        count = ((Long) q.getSingleResult()).intValue();

        return count;
    }

    private List<T> findEntities(boolean all, int maxResults, int firstResult) {
        List<T> found = new ArrayList<>();
        CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
        cq.select(cq.from(clazz));
        Query q = em.createQuery(cq);
        if (!all) {
            q.setMaxResults(maxResults);
            q.setFirstResult(firstResult);
        }
        found.addAll(q.getResultList());

        return found;
    }
}
