package edu.chl.hajo.jpaem;

import edu.chl.hajo.jpaem.core.Person;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.criteria.CriteriaQuery;

/**
 * Wrapper for database
 * @author hajo
 */
@Stateless  // A stateless EJB (with no interface)
public class PersonRegistry implements Serializable {

    private static final String PU = "database_pu";
    // This is JTA see persistence.xml
    @PersistenceContext(unitName = PU)
    private EntityManager em;

    public PersonRegistry(){
        
    }
    
    // Used for testing outside container (possible to supply an em
    public PersonRegistry(EntityManager em){
        this.em = em;
    }
    
    // Transactions handled by container
    public void insert(Person p) {
        em.persist(p);
    }

    public Person selectByPk(Long pnum) {
        return em.find(Person.class, pnum);
    }

    public Person update(Person p) {
        return em.merge(p);
    }

    public void delete(Long pnumb) {
        Person t = em.getReference(Person.class, pnumb);
        em.remove(t);
    }

    public List<Person> selectAll() {
        return select(true, 0, 0);
    }

    private List<Person> select(boolean all, int start, int count) {
        List<Person> persons = new ArrayList<>();

        CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
        cq.select(cq.from(Person.class));
        Query q = em.createQuery(cq);
        if (!all) {
            q.setMaxResults(count);
            q.setFirstResult(start);
        }
        persons.addAll(q.getResultList());
        return persons;
    }

    public int count() {
        // Not implemented
        return 0;
    }
}
